----------------------------------------------------------------------------------
-- Company: Digilent Inc.
-- Engineer: Andrew Skreen
-- 
-- Create Date:    17:25:25 03/01/2012 
-- Module Name:    PmodCLS_Demo
-- Project Name: 	 PmodCLS_Demo
-- Target Devices: Nexys3
-- Tool versions:  ISE 14.1
-- Description: Prints "Hello From Digilent" on PmodCLS connected to Nexys3
--					 on pins JA1-JA4. SPI protocol is utilized for communications
--					 between the PmodCLS and the Nexys3, hence the appropriate
--					 jumpers should be set on the PmodCLS, for details see
--					 PmodCLS reference manual.
--
-- Revision: 1.00 - Added debouncing, comments, fixed reset issue.
-- Revision: 0.01 - File Created
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- *******************************************************************************
--  								Define block inputs and outputs
-- *******************************************************************************
entity PmodCLS_Demo is
    Port ( sw : in  STD_LOGIC_VECTOR (2 downto 0);		-- sw[0] is reset
																		-- sw[1] is clear
																		-- sw[2] is start
           JA : inout  STD_LOGIC_VECTOR (3 downto 0); -- Port JA[4:1]
           Led : inout  STD_LOGIC_VECTOR (0 downto 0); -- Led<0>
           clk : in  STD_LOGIC);								-- Onboard 100 Mhz clock
end PmodCLS_Demo;

architecture Behavioral of PmodCLS_Demo is

-- *******************************************************************************
--  									Component Declarations
-- *******************************************************************************


		-- ++++++++++++++++++++++++++++++++++++++++++++++
		-- 					Master Interface
		-- ++++++++++++++++++++++++++++++++++++++++++++++
		component master_interface
			port ( begin_transmission : out std_logic;
					 end_transmission : in std_logic;
					 clk : in std_logic;
					 rst : in std_logic;
					 start : in std_logic;
					 clear : in std_logic;
					 slave_select : out std_logic;
					 sel : out integer range 0 to 33;
					 temp_data : in std_logic_vector(7 downto 0);
					 send_data : out std_logic_vector(7 downto 0));
		end component;

		-- ++++++++++++++++++++++++++++++++++++++++++++++
		-- 				Serial Port Interface
		-- ++++++++++++++++++++++++++++++++++++++++++++++
		component spi_interface
			port ( begin_transmission : in std_logic;
					 slave_select : in std_logic;
					 send_data : in std_logic_vector(7 downto 0);
					 miso : in std_logic;
					 clk : in std_logic;
					 rst : in std_logic;
					 --recieved_data : out std_logic_vector(7 downto 0);
					 end_transmission : out std_logic;
					 mosi : out std_logic;
					 sclk : out std_logic);
		end component;


		-- ++++++++++++++++++++++++++++++++++++++++++++++
		-- 					Command Lookup
		-- ++++++++++++++++++++++++++++++++++++++++++++++
		component command_lookup
			port( sel : in integer range 0 to 33;
					data_out : out std_logic_vector(7 downto 0));
		end component;
		
		
		-- ++++++++++++++++++++++++++++++++++++++++++++++
		-- 						Debouncer
		-- ++++++++++++++++++++++++++++++++++++++++++++++
		component three_bit_debouncer
			port( 
					CLK : in STD_LOGIC;							
					RST : in STD_LOGIC;							
					DIN : in STD_LOGIC_VECTOR(2 downto 0);
					DOUT : out STD_LOGIC_VECTOR(2 downto 0)
			);
		end component;


-- *******************************************************************************
--  									Signals and Constants
-- *******************************************************************************

	-- Active low signal for writing data to PmodCLS
	signal slave_select : std_logic;
	-- Initializes data transfer with PmodCLS
	signal begin_transmission : std_logic;
	-- Handshake signal to signify data transfer done
	signal end_transmission : std_logic;
	-- Selects which ASCII value to send to PmodCLS
	signal sel : integer range 0 to 33;
	-- Output data from C2 to C0
	signal temp_data : std_logic_vector(7 downto 0);
	-- Output data from C0 to C1
	signal send_data : std_logic_vector(7 downto 0);
	-- Ground, i.e. logic low
	signal GRND : std_logic := '0';
	-- dbSW[0] Debounced SW[0], dbSW[1] Debounced SW[1], dbSW[2] Debounced SW[2]
	signal dbSW : std_logic_vector(2 downto 0) := "000";

-- *******************************************************************************
--  										Implementation
-- *******************************************************************************
begin

	-- Debounces the input control signals from switches.
	DebounceInputs : Three_Bit_Debouncer port map(
			CLK=>CLK,
			RST=>GRND,
			DIN=>SW,
			DOUT=>dbSW
	);

	-- Produces signals for controlling SPI interface, and selecting output data.
	C0: master_interface port map( 
			begin_transmission => begin_transmission,
			end_transmission => end_transmission,
			clk => clk,
			rst => dbSW(0),
			start => dbSW(2),
			clear => dbSW(1),
			slave_select => slave_select,
			temp_data => temp_data,
			send_data => send_data,
			sel => sel
	);

	-- Interface between the PmodCLS and FPGA, proeduces SCLK signal, etc.
	C1 : spi_interface port map(
			begin_transmission => begin_transmission,
			slave_select => slave_select,
			send_data => send_data,
			--recieved_data => recieved_data,
			miso => JA(2),
			clk => clk,
			rst => dbSW(0),
			end_transmission => end_transmission,
			mosi => JA(1),
			sclk => JA(3)
	);

	-- Contains the ASCII characters for commands
	C2 : command_lookup port map (
			sel => sel,
			data_out => temp_data
	);

	--  Active low slave select signal
	JA(0) <= slave_select;

	--  Assign Led<0> the value of SW(0)
	Led(0) <= '1' when SW(0) = '1' else '0';

end Behavioral;

